package org.rcisoft.integration.jieLink.service.impl;


import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.integration.jieLink.common.SignUtils;
import org.rcisoft.integration.jieLink.dto.*;
import org.rcisoft.integration.jieLink.service.IJieLinkService;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.sysuser.service.SysUserService;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * 捷顺道闸对接接口实现
 */
@Slf4j
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class JieLinkServiceImpl  implements IJieLinkService {

    /**
     * 捷顺服务地址
     */
    private String jieLinkUri;

    /**
     * 接口用户
     */
    private String jieLinkUser;

    /**
     * 接口密码
     */
    private String jieLinkPwd;


    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private SysUserService sysUserService;




    /**
     * 获取签名所需相关参数
     * @return
     */
    @Override
    public SignDto getAppKeyInfo() {
        String apiUri ="/api/internal/sign";
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
        headers.set("Accept", MediaType.APPLICATION_JSON_VALUE);
        JSONObject paramMap = new JSONObject();
        paramMap.put("userName",jieLinkUser);
        paramMap.put("password",jieLinkPwd);
        HttpEntity<JSONObject> httpEntity = new HttpEntity<>(paramMap,headers);
        StringBuffer uriAddr = new StringBuffer();
        uriAddr.append(jieLinkUri).append(apiUri);
        try{
            ResponseEntity<String> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, String.class);
            if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                JieLinkResultDto<List<AppInfoDto>> resultData = JSONObject.parseObject(responseEntity.getBody(), JieLinkResultDto.class);
                if(StringUtils.equals(resultData.getCode(),"0")){
                    List<AppInfoDto> appInfoDtoList = resultData.getData();
                    if(appInfoDtoList!=null && !appInfoDtoList.isEmpty()){
                        AppInfoDto appInfoDto = appInfoDtoList.get(0);
                        SignDto signData = new SignDto();
                        signData.setAppId(appInfoDto.getAppId());
                        signData.setKey(appInfoDto.getKey());
                        return signData;
                    }else{
                        //返回结果为空
                        return null;
                    }

                }else{
                    log.error("getAppKeyInfo接口返回错误信息："+resultData.getMsg());
                    //请求返回结果错误啦
                    return null;
                }
            }else{
                //请求失败啦
                return null;
            }
        }catch (Exception ex){
            log.error(ex.getMessage(),ex);
            return null;
        }
    }

    /**
     * 推送新增用户对象到捷顺服务中
     * @param userData
     * @return
     */
    @Override
    public boolean pushAddUserData(SysUser userData) {
        String apiUri = "/api/base/addperson";
        HttpHeaders headers = this.loadHeaderHandle();
        if(headers!=null){
            JieLinkPersonDto pushData = this.convertUserBean(userData);
            HttpEntity<JieLinkPersonDto> httpEntity = new HttpEntity<>(pushData,headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<String> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, String.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                    JieLinkResultDto<JieLinkPersonDto> resultData = JSONObject.parseObject(responseEntity.getBody(), JieLinkResultDto.class);
                    if(StringUtils.equals(resultData.getCode(),"0")){
                        JieLinkPersonDto persion = resultData.getData();
                        if(persion!=null){
                            //此处需要将捷顺中的ID存入到用户信息中
                            String personId = persion.getPersonId();
                            if(userData.getBusinessId()!=null){
                                sysUserService.saveJieLinkId(personId,userData.getBusinessId());
                                return true;
                            }else{
                                //ID竟然没有返回，说明参数没有传全
                                log.error("pushAddUserData接口请求时缺少用户ID");
                                return false;
                            }
                        }else{
                            //返回结果为空
                            return false;
                        }

                    }else{
                        log.error("pushAddUserData接口返回错误信息："+resultData.getMsg());
                        //请求返回结果错误啦
                        return false;
                    }
                }else{
                    //请求失败啦
                    return false;
                }
            }catch (Exception ex){
                log.error(ex.getMessage(),ex);
                return false;
            }
        }else{
            //认证信息获取失败
            return false;
        }
    }


    /**
     * 推送更新的用户信息到接口服务中
     * @param userData
     * @return
     */
    @Override
    public boolean pushModifyUserData(SysUser userData) {
        String apiUri = "/api/base/updateperson";
        HttpHeaders headers = this.loadHeaderHandle();
        if(headers!=null){
            JieLinkPersonDto pushData = this.convertUserBean(userData);
            pushData.setPersonId("");//TODO 此处缺失关联捷顺系统的ID
            HttpEntity<JieLinkPersonDto> httpEntity = new HttpEntity<>(pushData,headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<String> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, String.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                    JieLinkResultDto<JieLinkPersonDto> resultData = JSONObject.parseObject(responseEntity.getBody(), JieLinkResultDto.class);
                    if(StringUtils.equals(resultData.getCode(),"0")){
                        return true;
                    }else{
                        log.error("pushAddUserData接口返回错误信息："+resultData.getMsg());
                        //请求返回结果错误啦
                        return false;
                    }
                }else{
                    //请求失败啦
                    return false;
                }
            }catch (Exception ex){
                log.error(ex.getMessage(),ex);
                return false;
            }
        }else{
            //认证信息获取失败
            return false;
        }
    }

    /**
     * 用户对象转换
     * @param userData
     * @return
     */
    private JieLinkPersonDto convertUserBean(SysUser userData){
        JieLinkPersonDto pushData = new JieLinkPersonDto();
        pushData.setPersonNo(userData.getUsername());//人员编号
        pushData.setPersonName(userData.getName());//姓名 必填
        pushData.setPersonGender(StringUtils.equals("0",userData.getSex())?1:StringUtils.equals("1",userData.getSex())?0:null);//性别
        pushData.setPersonPhoto(userData.getFaceAddress());
        //身份证处理
        if(StringUtils.isNotBlank(userData.getIdNumber())){
            if(userData.getIdNumber().trim().length()==15){
                pushData.setCertificateType("GENERIDENT");//一代身份证
            }else {
                //其他类型不符合长度要求，默认按照二代身份证处理
                pushData.setCertificateType("IDENTITY");//默认二代身份证
            }
            pushData.setIdentityNo(userData.getIdNumber());//身份证
        }
        pushData.setMobile(userData.getPhone());//必填
        pushData.setEmail(userData.getEmail());
        if(StringUtils.isNotBlank(userData.getUserType())){
            if(StringUtils.equals("0", userData.getUserType())){
                pushData.setTenementType(1);//正式员工
            }else {
                pushData.setTenementType(5);//非正式员工以外全部定义为临时人员
            }
        }
       return pushData;
    }


    /**
     * 推送方可邀请记录到接口服务中
     * @param visitInfoMation
     * @return
     */
    @Override
    public boolean pushVisitorData(VisitInfoMation visitInfoMation) {
        String apiUri = "/api/base/visitordatainvite";
        HttpHeaders headers = this.loadHeaderHandle();
        if(headers!=null){
            JSONObject pushData = this.convertVisitorData(visitInfoMation);
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(pushData,headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<String> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, String.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                        //后续处理逻辑
                    JieLinkResultDto<JSONObject> resultData = JSONObject.parseObject(responseEntity.getBody(), JieLinkResultDto.class);
                    if(StringUtils.equals(resultData.getCode(),"0")){
                        return true;
                    }else{
                        log.error("pushVisitorData接口返回错误信息："+resultData.getMsg());
                        //请求返回结果错误啦
                        return false;
                    }
                }else{
                    //请求失败啦
                    return false;
                }
            }catch (Exception ex){
                log.error(ex.getMessage(),ex);
                return false;
            }
        }else{
            //认证信息获取失败
            return false;
        }
    }


    /**
     * 撤销邀访记录
     * @param visitUserId 邀访人Id
     * @param visitInfoId 邀请记录ID
     * @return
     */
    @Override
    public boolean cancelVisitorData(String visitUserId, String visitInfoId) {
        String apiUri = "/api/base/visitdatacancelinvite";
        HttpHeaders headers = this.loadHeaderHandle();
        if(headers!=null){
            JSONObject pushData = new JSONObject();
            if(StringUtils.isNotBlank(visitInfoId)){
                pushData.put("inviteId",visitInfoId);
            }else if(StringUtils.isNotBlank(visitUserId)){
                pushData.put("visitorId",visitUserId);
            }else{
                //
                log.error("cancelVisitorData 接口传递的参数信息有误，请检查参数");
                return false;
            }
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(pushData,headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<String> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, String.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                    JieLinkResultDto resultData = JSONObject.parseObject(responseEntity.getBody(), JieLinkResultDto.class);
                    if(StringUtils.equals(resultData.getCode(),"0")){
                        return true;
                    }else{
                        log.error("cancelVisitorData接口返回错误信息："+resultData.getMsg());
                        //请求返回结果错误啦
                        return false;
                    }
                }else{
                    //请求失败啦
                    return false;
                }
            }catch (Exception ex){
                log.error(ex.getMessage(),ex);
                return false;
            }
        }else{
            //认证信息获取失败
            return false;
        }
    }

    /**
     * 邀访记录转换
     * @param infoMation
     * @return
     */
    private JSONObject convertVisitorData(VisitInfoMation infoMation){
        JSONObject result = new JSONObject();
        VisitDataInviteDto manData = new VisitDataInviteDto();
        //TODO 此处需要沟通下才行
        manData.setPersonId(sysUserService.getPersionIdByUserId(infoMation.getUserId()));//转换ID
        manData.setItemId(infoMation.getItemId());//邀访记录
        //生成邀访时间格式的JSON字符串
        manData.setTimeDesc("");//邀访时间格式  需要根据两个字段以及数据字典中对应的类型进行计算得到开始和结束时间
        manData.setVisitorType("3");//访客类型
        //到访事由   两个字段做或判断
        if(StringUtils.isNotBlank(infoMation.getVisitReason())){
            manData.setContent(infoMation.getVisitReason());
        }else{
            manData.setContent(infoMation.getVisitReasonOther());
        }

        List<VisitDataSubItemDto> userList = new ArrayList<>();
        if(infoMation.getUserList()!=null && !infoMation.getUserList().isEmpty()){
            List<VisitUser> userDataList = infoMation.getUserList();
            for(VisitUser user: userDataList){
                VisitDataSubItemDto userItemDto = new VisitDataSubItemDto();
                //此处的访客ID存在问题，一会沟通下
                userItemDto.setVisitorId(user.getPersonId());//访客ID
                userItemDto.setVisitorName(user.getUserName());//访客名称
                userItemDto.setVisitorTel(user.getVisitPhone());//访客手机号
                //下面需要构建凭证信息，此处没有用数据获取的源头
                List<VisitDataCredentialDto> credentialDtos = new ArrayList<>();
                VisitDataCredentialDto credentialDto = new VisitDataCredentialDto();
                credentialDto.setCredentialId(UUID.randomUUID().toString());
                credentialDto.setCredentialType("1");
                String cardNo = RandomUtil.randomString("ABCDEF0123456789", 8);
                credentialDto.setCredentialNo(cardNo);
                credentialDtos.add(credentialDto);
                userItemDto.setCredentialList(credentialDtos);
                ///参数对应啦
                userList.add(userItemDto);
            }
        }
        result.put("visitDataInviteModel",manData);
        result.put("visitDataSubItemModelList",userList);
        return result;
    }


    /***
     * 拉取访客所需的二维码信息
     * @return
     */
    @Override
    public String pullVisitUserQrCode(String visitUserId) {
        String apiUri = "/api/base/getqrcodedata";
        HttpHeaders headers = this.loadHeaderHandle();
        if(headers!=null){
            JSONObject pushData = new JSONObject();
            pushData.put("guid",visitUserId);
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(pushData,headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<String> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, String.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                    JieLinkResultDto<JSONObject> resultData = JSONObject.parseObject(responseEntity.getBody(), JieLinkResultDto.class);
                    if(StringUtils.equals(resultData.getCode(),"0")){
                        JSONObject data = resultData.getData();
                        if(data.containsKey("qrCodeData")){
                            return data.getString("qrCodeData");
                        }else{
                            return null;
                        }
                    }else{
                        log.error("cancelVisitorData接口返回错误信息："+resultData.getMsg());
                        //请求返回结果错误啦
                        return null;
                    }
                }else{
                    //请求失败啦
                    return null;
                }
            }catch (Exception ex){
                log.error(ex.getMessage(),ex);
                return null;
            }
        }else{
            //认证信息获取失败
            return null;
        }
    }


    /**
     * 保存门禁访问记录
     * @param recordDto
     * @return
     */
    @Override
    public JieLinkResultDto saveDoorRecordInfo(JieLinkDoorRecordDto recordDto) {
        /**
         * 开始处理上传的门禁记录
         * 1、检查访客人员是否存在，不存在是否还需要新增
         * 2、检查设备系统内是否存在，不存在返回错误信息
         * 3、保存门禁记录信息，按照年划分
         */
        JieLinkResultDto result = new JieLinkResultDto();
        JieLinkPersonDto person = recordDto.getPerson();
        if(person!=null){//

        }else{
            //访客记录为空了
        }

        return null;
    }


    /**
     * 创建hander头信息
     * @return
     */
    private HttpHeaders loadHeaderHandle(){
        //加载头部信息
        if(SignUtils.signData!=null){
            SignDto signData = SignUtils.getSignData(SignUtils.signData);
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
            headers.set("Accept", MediaType.APPLICATION_JSON_VALUE);
            headers.set("appId",signData.getAppId());
            headers.set("v",signData.getV());
            headers.set("random",signData.getRandom());
            headers.set("timestamp",signData.getTimestamp());
            headers.set("sign",signData.getSign());
            return headers;
        }else{
            SignDto appKeyInfo = this.getAppKeyInfo();
            if(appKeyInfo!=null){
                SignUtils.signData = appKeyInfo;
                SignDto signData = SignUtils.getSignData(appKeyInfo);
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
                headers.set("Accept", MediaType.APPLICATION_JSON_VALUE);
                headers.set("appId",signData.getAppId());
                headers.set("v",signData.getV());
                headers.set("random",signData.getRandom());
                headers.set("timestamp",signData.getTimestamp());
                headers.set("sign",signData.getSign());
                return headers;
            }else{
                return null;
            }
        }
    }


}
