package org.rcisoft.sys.visit.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.common.component.CyGlobal;
import org.rcisoft.core.component.CyDownErrorTxtComp;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.jwt.util.CyJwtUtil;
import org.rcisoft.core.result.enums.CyResSvcExcEnum;
import org.rcisoft.core.util.CyCompareToolUtils;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.dto.ExportUserDTO;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.sysuser.service.impl.SysUserServiceImpl;
import org.rcisoft.sys.sysuser.util.CheckUtil;
import org.rcisoft.sys.sysuser.util.CyEpExcelUtil;
import org.rcisoft.sys.visit.dao.VisitInfomationRepository;
import org.rcisoft.sys.visit.dao.VisitUserRepository;
import org.rcisoft.sys.visit.dto.ExportVisitInfoDto;
import org.rcisoft.sys.visit.dto.ImportVistInfoDTO;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitInfomationService;
import org.rcisoft.sys.wbac.user.dto.ErrorDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Created by cy on 2022年11月16日 下午2:40:21.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class VisitInfomationServiceImpl extends ServiceImpl<VisitInfomationRepository, VisitInfoMation> implements VisitInfomationService {

    @Autowired
    VisitUserRepository visitUserRepository;

    @Autowired
    SysUserRepositorys sysUserRepositorys;

    @Autowired
    VisitUserServiceImpl userService;

    @Autowired
    private CyJwtUtil cyJwtUtil;

    @Autowired
    private CyGlobal global;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Autowired
    private CyDownErrorTxtComp errorTxt;

    @Autowired
    private SysUserServiceImpl sysUserService;

    /**
     * 保存 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(VisitInfoMation visitInfomation) {
        //增加操作
        if (CollectionUtils.isEmpty(visitInfomation.getUserList()))
            throw new CyServiceException(500, "请至少新增一名访客");
        visitInfomation.setNormal();
        String[] regions=visitInfomation.getVisitRegions();
        String region = "";
        for (String r:regions) {
            region = region +r +',';
        }
        region = region.substring(0, region.length() - 1);
        visitInfomation.setVisitRegion(region);

        String[] regionCodes = visitInfomation.getVisitRegionCodes();
        String regionCode = "";
        for (String code:regionCodes){
            regionCode = regionCode + code +",";
        }
        if (regionCode.length()>0){
            regionCode = regionCode.substring(0,regionCode.length()-1);
            visitInfomation.setVisitRegionCode(regionCode);
        }
        int line = baseMapper.insert(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息信息");
        // 添加访客信息
        visitInfomation.getUserList().forEach(u -> {
            // 判断当前访客(用户)是否已经存在
            SysUser sysUser = new SysUser();
            sysUser.setName(u.getUserName());
            sysUser.setPhone(u.getVisitPhone());
            sysUser.setUsername(u.getVisitPhone());
            sysUser.setPassword(passwordEncoder.encode(global.getResetPassword()));
            sysUser.setNormal();
            List<SysUser> sysUserList = sysUserRepositorys.querySysUsers(sysUser);
            // 不存在新增
            if (CollectionUtils.isEmpty(sysUserList)) {
                sysUserRepositorys.insert(sysUser);
            } else {
                // 存在覆盖
                sysUser.setBusinessId(sysUserList.get(0).getBusinessId());
                sysUserRepositorys.updateSysUser(sysUser);
            }
            u.setVisitId(visitInfomation.getBusinessId());
            u.setNormal();
            visitUserRepository.insert(u);
        });
        return new CyPersistModel(line);
    }

    /**
     * 删除 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(VisitInfoMation visitInfomation) {
        int line = baseMapper.realDelete(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(VisitInfoMation visitInfomation) {
        visitInfomation.setDeleted();
        int line = baseMapper.deleteById(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(VisitInfoMation visitInfomation) {
        List<VisitUser> visitUsers = visitInfomation.getUserList();

        if (visitUsers.size() > 0) {
            userService.deleteByVisitId(visitInfomation.getBusinessId());
            for (VisitUser visitUser : visitUsers) {
                userService.persist(visitUser);
            }
        }
        int line = baseMapper.updateById(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + visitInfomation.getBusinessId() + "的邀访信息信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 邀访信息
     *
     * @param id
     * @return
     */
    @Override
    public VisitInfoMation findById(int id) {
        VisitInfoMation v = baseMapper.queryVisitById(id);
        if (v.getVisitRegionCode() !=null){
            String[] regionCodes = v.getVisitRegionCode().split(",");
            v.setVisitRegionCodes(regionCodes);
        }
        if (v.getVisitRegion() != null) {
            String[] regions = v.getVisitRegion().split(",");
            v.setVisitRegions(regions);
        }
        String vuList = "";
        for (VisitUser vu : v.getUserList()) {
            vuList = vuList + vu.getUserName() + ',';
        }
        if (vuList.length()>0){
            vuList = vuList.substring(0, vuList.length() - 1);
        }
        v.setUserArry(vuList);
        return v;
    }

    /**
     * 分页查询 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Override
    public IPage<VisitInfoMation> findAllByPagination(CyPageInfo<VisitInfoMation> paginationUtility,
                                                      VisitInfoMation visitInfomation) {
        if (visitInfomation.getVisitReason()!= null && visitInfomation.getVisitReason().length()>0){
            String[] reasons = visitInfomation.getVisitReason().split(",");
            String[] reasonCodes = visitInfomation.getVisitReasonCode().split(",");
            for (String r: reasons) {
                if (r.equals("other")){
                    visitInfomation.setBoolExistOtherReason("YES");
                }
            }
            visitInfomation.setVisitReasonCodes(reasonCodes);
            visitInfomation.setVisitReasons(reasons);
        }
        if (visitInfomation.getVisitRegion()!= null && visitInfomation.getVisitRegion().length()>0){
            String[] regions = visitInfomation.getVisitRegion().split(",");
            String[] regionCodes = visitInfomation.getVisitRegionCode().split(",");
            visitInfomation.setVisitRegionCodes(regionCodes);
            visitInfomation.setVisitRegions(regions);
        }
        IPage<VisitInfoMation> list = baseMapper.queryVisitInfomationsPaged(paginationUtility, visitInfomation);
        List<VisitInfoMation> mationList = list.getRecords();
        for (VisitInfoMation v : mationList) {
            if (v.getVisitRegionCode() !=null){
                String[] regionCodes = v.getVisitRegionCode().split(",");
                v.setVisitRegionCodes(regionCodes);
            }
            if (v.getVisitRegion() != null) {
                String[] regions = v.getVisitRegion().split(",");
                v.setVisitRegions(regions);
            }
            String vuList = "";
            for (VisitUser vu : v.getUserList()) {
                vuList = vuList + vu.getUserName() + ',';
            }
            if (vuList.length()>0){
                vuList = vuList.substring(0, vuList.length() - 1);
            }
            v.setUserArry(vuList);
        }
        list.setRecords(mationList);
        System.out.println(list);
        return baseMapper.queryVisitInfomationsPaged(paginationUtility, visitInfomation);

    }

    /**
     * 查询list 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Override
    public List<VisitInfoMation> findAll(VisitInfoMation visitInfomation) {
        List<VisitInfoMation> visitInfomationList = baseMapper.queryVisitList(visitInfomation);
        for (VisitInfoMation v : visitInfomationList) {
            String vuList = "";
            for (VisitUser vu : v.getUserList()) {
                vuList = vuList + vu.getUserName() + ',';
            }
            vuList = vuList.substring(0, vuList.length() - 1);
            v.setUserArry(vuList);
        }
        return visitInfomationList;
    }

    /**
     * 导出邀访信息
     *
     * @return
     */
    @Override
    public List<VisitInfoMation> export(VisitInfoMation visitInfomation) {
        List<VisitInfoMation> visitInfomationList = baseMapper.queryVisitList(visitInfomation);
        for (VisitInfoMation v : visitInfomationList) {
            String vuList = "";
            for (VisitUser vu : v.getUserList()) {
                vuList = vuList + vu.getUserName() + ',';
            }
            if (vuList.length()>0){
                vuList = vuList.substring(0, vuList.length() - 1);
            }
            v.setUserArry(vuList);
        }
        return visitInfomationList;
    }

    /**
     * 获取当前用户最新受邀记录
     *
     * @return
     */
    @Override
    public VisitInfoMation lastDetail(HttpServletRequest request) {
        // 获取当前登录用户的手机号
        String phone = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_PHONE);
        if (StringUtils.isEmpty(phone))
            return new VisitInfoMation();
        // 获取当前登录用户的受邀记录
        List<VisitInfoMation> visitInfList = baseMapper.lastDetail(phone);
        if (CollectionUtils.isEmpty(visitInfList) || visitInfList.size() == 0)
            return new VisitInfoMation();
        else
            return visitInfList.get(0);
    }

    @Override
    public List<VisitInfoMation> getPastOrFutureInfo() {

        List<VisitInfoMation> resList = new ArrayList<>();

        List<VisitInfoMation> pastList = baseMapper.queryPast();
        int pastListLength = pastList.size();
        List<VisitInfoMation> futureList = baseMapper.queryFuture();
        int futureListLength = futureList.size();

        if (pastListLength > 0 && futureListLength > 0) {
            resList.add(futureList.get(0));
            resList.add(pastList.get(0));
            return resList;
        }
        if (pastListLength == 0 && futureListLength > 1) {
            resList.add(futureList.get(1));
            resList.add(futureList.get(0));
            return resList;
        }
        if (pastListLength == 0 && futureListLength == 1) {
            resList.add(futureList.get(0));
            return resList;
        }
        if (pastListLength > 1 && futureListLength == 0) {
            resList.add(pastList.get(0));
            resList.add(pastList.get(1));
            return resList;
        }
        if (pastListLength == 1 && futureListLength == 0) {
            resList.add(pastList.get(0));
            return resList;
        }
        return resList;
    }

    @Override
    public int exportEmptyTemplate(HttpServletResponse response) {
        List<ExportVisitInfoDto> imOutSysUsers = new ArrayList<>();
        ExportVisitInfoDto exportUserDTO = new ExportVisitInfoDto();
        for (int i = 0; i < 500; i++) {
            imOutSysUsers.add(exportUserDTO);
        }
        CyEpExcelUtil.exportExcelSelect(imOutSysUsers, "邀访信息", "邀访",
                ExportVisitInfoDto.class, "模板.xls", response, null);
        return 1;
    }

    @Override
    public ImportVistInfoDTO importUserExcel(MultipartFile file) throws ParseException {
        ImportVistInfoDTO importVistInfoDTO = new ImportVistInfoDTO();
        Calendar calendar = Calendar.getInstance();
        String nowTime = new SimpleDateFormat("yyyyMMddHHmmss").format(calendar.getTime());
        String fileName = "邀访错误信息" + nowTime + ".txt";
        List<Map<Integer, List<String>>> maps = new ArrayList<>();
        ExportVisitInfoDto exportVisitInfoDto = new ExportVisitInfoDto();
        boolean flag = false;
        try {
            flag = CyEpExcelUtil.isFileInUser(file, exportVisitInfoDto, 2);
        } catch (Exception e) {
            throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
        }
        if (!flag || !CyCompareToolUtils.compareFileName(file, "邀访信息", 0, 0)) {
            String errorBefore = "上传模板错误请重新下载！";
            errorText(errorBefore, importVistInfoDTO, maps, fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importVistInfoDTO;
        }
        /** 从excel表中读到数据，有一部分为空值 */
        List<ExportVisitInfoDto> visitInfoDtoList = CyEpExcelUtil.importExcel(file, 2, 1, ExportVisitInfoDto.class);
        /** 校验后没有问题的数据 */
        List<ExportVisitInfoDto> addINfoList = new ArrayList<>();
        /** 遍历 从Excel表中读到的数据 抛出不合适的数据 */
        //判断整个Excel是不是为空 计算非空数据有多少条
        int information = CyCompareToolUtils.cmpareInformation(visitInfoDtoList);
        if (information == 0) {
            String errorBefore = "Excel文件不可为空，请检查Excel文件是否填写！";
            errorText(errorBefore, importVistInfoDTO, maps, fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importVistInfoDTO;
        } else if (information > 500) {
            String errorMore = "单次导入数据不可超过500，请检查！";
            errorText(errorMore, importVistInfoDTO, maps, fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importVistInfoDTO;
        } else {
            //去除空数据
            try {
                visitInfoDtoList = sysUserService.ThrowNull(visitInfoDtoList);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            //校验 将错误信息放入maps
            int index = 3; //从第3行开始
            for (ExportVisitInfoDto visitInfoDto : visitInfoDtoList) {
                //判断 是否为空
                ErrorDTO infoNull = CheckUtil.infoNotNull(visitInfoDto, index);
                if (infoNull.getSuccess() == 1)
                    maps.add(infoNull.getMap());
                else
                    addINfoList.add(visitInfoDto);
                index++;
            }
        }
        if (addINfoList.size() > 0) {
            for (ExportVisitInfoDto visitInfoDto : addINfoList) {
                VisitInfoMation visitInfoMation = new VisitInfoMation();
                // 根据姓名查询
                SysUser sysUserDTO = new SysUser();
                sysUserDTO.setDeleted();
                sysUserDTO.setName(visitInfoDto.getUserName());
                sysUserDTO.setUserType("0");
                // 根据姓名查询员工数据
                List<SysUser> sysUsers = sysUserRepositorys.querySysUsers(sysUserDTO);
                if (sysUsers.size() == 0)
                    continue;
                visitInfoMation.setUserId(sysUsers.get(0).getBusinessId());
                visitInfoMation.setUserName(visitInfoDto.getUserName());
                visitInfoMation.setDeptName(visitInfoDto.getDeptName());
                visitInfoMation.setVisitRegion(visitInfoDto.getVisitRegion());
                visitInfoMation.setVisitReason(visitInfoDto.getVisitReason());
                SimpleDateFormat date = new SimpleDateFormat("yyyy-MM-dd");
                visitInfoMation.setVisitDate(date.parse(visitInfoDto.getVisitDate()));
                visitInfoMation.setVisitTime(visitInfoDto.getVisitTime());
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                String time = visitInfoDto.getVisitDate().toString() + " " + visitInfoDto.getVisitTime();
                visitInfoMation.setVisitRealTime(dateFormat.parse(time));
                List<VisitUser> userList = new ArrayList<>();
                // 判断访客姓名和手机号是否数量匹配
                List<String> visitName = Arrays.asList(visitInfoDto.getVisitName().split(",")).stream()
                        .filter(u -> StringUtils.isNotEmpty(u)).collect(Collectors.toList());
                List<String> visitPhone = Arrays.asList(visitInfoDto.getVisitPhone().split(",")).stream()
                        .filter(p -> StringUtils.isNotEmpty(p)).distinct()
                        .collect(Collectors.toList());
                for (int i = 0; i < visitName.size(); i++) {
                    userList.add(new VisitUser(visitName.get(i), visitPhone.get(i)));
                }
                visitInfoMation.setUserList(userList);
                // 新增
                persist(visitInfoMation);
            }
        }
        if (null != maps && maps.size() > 0) {
            importVistInfoDTO.setFilename(fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
        }
        return importVistInfoDTO;
    }


    void errorText(String error, ImportVistInfoDTO importVistInfoDTO, List<Map<Integer, List<String>>> maps, String fileName) {
        List<String> strings = new ArrayList<String>();
        strings.add(error);
        Map<Integer, List<String>> map = new HashMap<>();
        map.put(-1, strings);
        maps.add(map);
        importVistInfoDTO.setFilename(fileName);
    }
}
