package org.rcisoft.sys.visit.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.component.CyDownErrorTxtComp;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.result.enums.CyResSvcExcEnum;
import org.rcisoft.core.util.CyCompareToolUtils;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.sys.dictionary.service.DictionaryService;
import org.rcisoft.sys.sysuser.dao.SysDeptRepositorys;
import org.rcisoft.sys.sysuser.dao.SysDictDataRepositorys;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.sysuser.service.impl.SysUserServiceImpl;
import org.rcisoft.sys.sysuser.util.CheckUtil;
import org.rcisoft.sys.sysuser.util.CyEpExcelUtil;
import org.rcisoft.sys.visit.dao.VisitUserRepository;
import org.rcisoft.sys.visit.dto.ExportVisitUserDTO;
import org.rcisoft.sys.visit.dto.ImportVistUserDTO;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitUserService;

import org.rcisoft.sys.wbac.user.dto.ErrorDTO;
import org.rcisoft.sys.wbac.user.dto.ExcelDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.text.SimpleDateFormat;
import java.util.*;

import lombok.extern.slf4j.Slf4j;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

/**
 * Created by cy on 2022年11月16日 下午4:30:18.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class VisitUserServiceImpl extends ServiceImpl<VisitUserRepository, VisitUser> implements VisitUserService {


    @Autowired
    private SysUserRepositorys sysUserRepositorys;
    @Autowired
    private SysDeptRepositorys sysDeptRepositorys;
    @Autowired
    private SysDictDataRepositorys sysDictDataRepositorys;
    @Autowired
    private DictionaryService dictionaryService;

    @Autowired
    private CyDownErrorTxtComp errorTxt;
    @Autowired
    private SysUserServiceImpl sysUserService;

    /**
     * 保存 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(VisitUser visitUser) {
        //增加操作
        int line = baseMapper.insert(visitUser);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                visitUser.getBusinessId() + "的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(VisitUser visitUser) {
        int line = baseMapper.realDelete(visitUser);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                visitUser.getBusinessId() + "的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(VisitUser visitUser) {
        visitUser.setDeleted();
        int line = baseMapper.deleteById(visitUser);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                visitUser.getBusinessId() + "的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(VisitUser visitUser) {
        int line = baseMapper.updateById(visitUser);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + visitUser.getBusinessId() + "的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 邀访访客信息表
     *
     * @param id
     * @return
     */
    @Override
    public VisitUser findById(int id) {
        return baseMapper.selectById(id);
    }

    /**
     * 分页查询 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Override
    public IPage<VisitUser> findAllByPagination(CyPageInfo<VisitUser> paginationUtility,
                                                VisitUser visitUser) {
        return baseMapper.queryVisitUsersPaged(paginationUtility, visitUser);

    }

    /**
     * 查询list 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Override
    public List<VisitUser> findAll(VisitUser visitUser) {
        return baseMapper.queryVisitUsers(visitUser);

    }

    /**
     * 导出邀访访客信息表
     *
     * @return
     */
    @Override
    public List<VisitUser> export(VisitUser visitUser) {
        List<VisitUser> visitUserList = baseMapper.queryVisitUsers(visitUser);
        return visitUserList;
    }

    @Override
    public int deleteByVisitId(int id) {
        return baseMapper.deleteByVisitId(id);
    }

    @Override
    public int exportEmptyTemplate(HttpServletResponse response) {
        List<ExportVisitUserDTO> dtoList = new ArrayList<>();
        ExportVisitUserDTO exportVisitUserDTO = new ExportVisitUserDTO();
        for (int i = 0; i < 500; i++) {
            dtoList.add(exportVisitUserDTO);
        }
        ExcelDTO excelDto = new ExcelDTO();
        CyEpExcelUtil.exportExcelSelect(dtoList, "访客信息", "访客",
                ExportVisitUserDTO.class, "模板.xls", response, excelDto);
        return 1;
    }

    void errorText(String error, ImportVistUserDTO importUserDTO, List<Map<Integer, List<String>>> maps, String fileName) {
        List<String> strings = new ArrayList<String>();
        strings.add(error);
        Map<Integer, List<String>> map = new HashMap<>();
        map.put(-1, strings);
        maps.add(map);
        importUserDTO.setFilename(fileName);
    }

    @Override
    public ImportVistUserDTO importVisitUserExcel(MultipartFile file) {
        ImportVistUserDTO importUserDTO = new ImportVistUserDTO();
        Calendar calendar = Calendar.getInstance();
        String nowTime = new SimpleDateFormat("yyyyMMddHHmmss").format(calendar.getTime());
        String fileName = "用户错误信息" + nowTime + ".txt";
        List<Map<Integer, List<String>>> maps = new ArrayList<>();
        ExportVisitUserDTO exportUserDTO = new ExportVisitUserDTO();
        boolean flag = false;
        try {
            flag = CyEpExcelUtil.isFileInUser(file, exportUserDTO, 2);
        } catch (Exception e) {
            throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
        }
        if (!flag || !CyCompareToolUtils.compareFileName(file, "访客信息", 0, 0)) {
            String errorBefore = "上传模板错误请重新下载！";
            errorText(errorBefore, importUserDTO, maps, fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importUserDTO;
        }
        /** 从excel表中读到数据，有一部分为空值 */
        List<ExportVisitUserDTO> sysUserList = CyEpExcelUtil.importExcel(file, 2, 1, ExportVisitUserDTO.class);
        /** 遍历 从Excel表中读到的数据 抛出不合适的数据 */
        //判断整个Excel是不是为空 计算非空数据有多少条
        int information = CyCompareToolUtils.cmpareInformation(sysUserList);
        if (information == 0) {
            String errorBefore = "Excel文件不可为空，请检查Excel文件是否填写！";
            errorText(errorBefore, importUserDTO, maps, fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importUserDTO;
        } else if (information > 500) {
            String errorMore = "单次导入数据不可超过500，请检查！";
            errorText(errorMore, importUserDTO, maps, fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importUserDTO;
        } else {
            //去除空数据
            try {
                sysUserList = sysUserService.ThrowNull(sysUserList);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            //校验 将错误信息放入maps
            int index = 3; //从第3行开始
            for (ExportVisitUserDTO sysUser : sysUserList) {
                //判断 是否为空
                ErrorDTO userNull = CheckUtil.visvitNotNull(sysUser, index);
                if (userNull.getSuccess() == 1) {
                    maps.add(userNull.getMap());
                }
                index++;
            }
        }
        /** 将 信息插入sys_user  */
        if (sysUserList.size() > 0) {
            for (ExportVisitUserDTO sysUser : sysUserList) {
                //用户类型
                SysUser user = new SysUser();
                user.setUserType("1");
                user.setPhone(sysUser.getPhone());
                // 判断当前访客是否存在
                List<SysUser> sysUsers = sysUserRepositorys.querySysUsers(user);
                user.setName(sysUser.getName());
                user.setUsername(sysUser.getPhone());
                if (sysUsers.size() == 0)
                    sysUserRepositorys.insert(user);
                else {
                    sysUsers.get(0).setName(sysUser.getName());
                    sysUserRepositorys.updateSysUser(sysUsers.get(0));
                }
            }
        }
        return importUserDTO;
    }
}
